<?php
/**
 * @package	HikaShop for Joomla! TGLZarinPal Plugin
 * @version	1.0.0
 * @author	TGLGroup <tglgroup.ir>
 * @developer Morteza Fard Saffari
 * @copyright Copyright 2021-2030 tglgroup.ir. All rights reserved.
 * @license	MIT License
 */
defined('_JEXEC') or die('Restricted access');

class plgHikashoppaymentTGLZarinpal extends hikashopPaymentPlugin
{
    protected $autoloadLanguage = true;

    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);
        //$this->loadLanguage();
    }

    public $accepted_currencies = array(
        'IRR','IRT'
    );

    public $multiple = true;
    public $name = 'tglzarinpal';
    public $doc_form = 'tglzarinpal';

    public $pluginConfig = array(
        'merchantId' => array('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MERCHANT_ID', 'input'),
        'type' => array('PLG_HIKASHOPPAYMENT_TGLZARINPAL_TYPE', 'radio', array(
            'normal' => 'PLG_HIKASHOPPAYMENT_TGLZARINPAL_TYPE_NORMAL',
            'zaringate' => 'PLG_HIKASHOPPAYMENT_TGLZARINPAL_TYPE_ZARINGATE',
        )),
        'test' => array('PLG_HIKASHOPPAYMENT_TGLZARINPAL_TEST', 'boolean', 0),
        'invalid_status' => array('INVALID_STATUS', 'orderstatus'),
        'verified_status' => array('VERIFIED_STATUS', 'orderstatus'),
        'debug' => array('DEBUG', 'boolean', 0),
    );

    public $cachedDebug = '';

    public function onBeforeOrderCreate(&$order,&$do)
    {
        if(parent::onBeforeOrderCreate($order, $do) === true)
            return true;


        if (empty($this->payment_params->merchantId))
        {
            $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_CHECK_CONFIGURATION'));
            $do = false;
            return false;
        }
        return true;
    }

    public function onAfterOrderConfirm(&$order, &$methods, $method_id) {

        parent::onAfterOrderConfirm($order, $methods, $method_id);

        if($this->currency->currency_locale['int_frac_digits'] > 0)
            $this->currency->currency_locale['int_frac_digits'] = 0;

        $returnUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=notify&notif_payment='.$this->name.'&tmpl=component&lang='.$this->locale.'&order_id='.$order->order_id. $this->url_itemid;
        $cancelUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$order->order_id . $this->url_itemid;

        try
        {
            if(!empty($this->payment_params->rm))
                $this->payment_params->rm = 2;

            $price = $order->cart->full_total->prices[0]->price_value_with_tax;

            if($this->currency->currency_code === 'IRR')
                $price = $price / 10;

            if(strpos($price, $this->currency->currency_locale['mon_decimal_point']) !== false)
                $price = substr($price, 0, strpos($price, $this->currency->currency_locale['mon_decimal_point']));

            $requestParams = array(
                'MerchantID' => $this->payment_params->merchantId ?? '',
                'Amount' => $price,
                'Description' => $order->order_id,
                'Email' => $this->user->user_email,
                'Mobile' => '',//$order->cart->billing_address->address_telephone ?? '',
                'CallbackURL' => $returnUrl
            );

            if($this->payment_params->test)
            {
                $webserviceUri = 'https://sandbox.zarinpal.com/pg/services/WebGate/wsdl';
                $webserviceActionUri = 'https://sandbox.zarinpal.com/pg/StartPay/{authority}';
            }
            else
            {
                $gatewayType = $this->payment_params->type == 'zaringate' ? 'zaringate' : 'normal';

                $webserviceUri = 'https://www.zarinpal.com/pg/services/WebGate/wsdl';
                if($gatewayType == 'zaringate')
                    $webserviceActionUri = 'https://www.zarinpal.com/pg/StartPay/{authority}/ZarinGate';
                else
                    $webserviceActionUri = 'https://www.zarinpal.com/pg/StartPay/{authority}';
            }

            if($this->payment_params->debug)
                $this->writeToLog($requestParams);

            $client = new SoapClient($webserviceUri, ['encoding' => 'UTF-8']);
            $result = $client->PaymentRequest($requestParams);

            if ($result->Status == 100)
            {
                $redirectUrl = str_replace('{authority}', $result->Authority, $webserviceActionUri);
                $this->app->redirect($redirectUrl);

                return true;
            }
            else
            {
                $this->app->enqueueMessage($this->getErrorMsg($result->Status), 'error');
                $this->app->redirect($cancelUrl);
            }
        }
        catch(SoapFault|Exception $e)
        {
            $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_FAILED_PAYMENT'), 'error');
            $this->app->redirect($cancelUrl);
        }

        return false;
    }

    public function onPaymentNotification(&$statuses)
    {
        try
        {
            $orderId = hikaInput::get()->getInt('order_id', 0);
            $cancelUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id=' . $orderId;

            if($orderId > 0)
            {
                $order = $this->getOrder($orderId);

                $this->loadPaymentParams($order);
                if(empty($this->payment_params))
                    return false;

                $this->loadOrderData($order);

                if(empty($order))
                {
                    $this->writeToLog('Could not load any order for your notification ' . $orderId);
                    return false;
                }

                if($this->payment_params->debug)
                    $this->writeToLog($order);

                $history = new stdClass();
                $history->notified = 0;
                $history->data = $this->writeToLog();

                $returnUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=checkout&task=after_end&order_id='.$order->order_id . $this->url_itemid;
                $cancelUrl = HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id='.$order->order_id . $this->url_itemid;
                $url = HIKASHOP_LIVE.'administrator/index.php?option=com_hikashop&ctrl=order&task=edit&order_id=' . $order->order_id;
                $orderText = "\r\n" . JText::sprintf('NOTIFICATION_OF_ORDER_ON_WEBSITE', $order->order_number, HIKASHOP_LIVE);
                $orderText .= "\r\n" . str_replace('<br/>', "\r\n", JText::sprintf('ACCESS_ORDER_WITH_LINK', $url, $url));

                // Get payment status
                $status = hikaInput::get()->getString('Status', '');
                if($status == 'OK')
                {
                    $authority = hikaInput::get()->getString('Authority', '');

                    $price = $order->order_full_price;
                    if($this->currency->currency_code === 'IRR')
                        $price = $price / 10;

                    if(strpos($price, $this->currency->currency_locale['mon_decimal_point']) !== false)
                        $price = substr($price, 0, strpos($price, $this->currency->currency_locale['mon_decimal_point']));

                    if($this->payment_params->test)
                        $webserviceUri = 'https://sandbox.zarinpal.com/pg/services/WebGate/wsdl';
                    else
                        $webserviceUri = 'https://www.zarinpal.com/pg/services/WebGate/wsdl';

                    // Request parameters
                    $requestParams = array(
                        'MerchantID' => $this->payment_params->merchantId ?? '',
                        'Authority' => $authority,
                        'Amount' => $price
                    );

                    $client = new SoapClient($webserviceUri, ['encoding' => 'UTF-8']);
                    $result = $client->PaymentVerification($requestParams);

                    if($this->payment_params->debug)
                        $this->writeToLog($result);

                    $history->data = $this->writeToLog();

                    if ($result->Status == 100)
                    {
                        // Success
                        if($order->order_status !== $this->payment_params->verified_status)
                        {
                            $email = new stdClass();
                            $email->subject = JText::sprintf('PAYMENT_NOTIFICATION_FOR_ORDER','TGLZarinPal',$result->Status,$order->order_number);
                            $email->body = str_replace('<br/>',"\r\n",JText::sprintf('PAYMENT_NOTIFICATION_STATUS','TGLZarinPal',$result->Status)).' '.JText::sprintf('ORDER_STATUS_CHANGED',$this->payment_params->verified_status)."\r\n\r\n".$orderText;

                            $history = new stdClass();
                            $history->notified = 1;
                            $history->amount = $price.$order->order_currency_info->currency_code;
                            $history->data = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_TRACKING_CODE') . $result->RefID;

                            $this->modifyOrder($orderId, $this->payment_params->verified_status, $history, $email);
                        }

                        $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_SUCCESSFUL_PAYMENT') . '<br>' . JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_TRACKING_CODE') . $result->RefID, 'success');
                        $this->app->redirect($returnUrl);

                        return true;
                    }
                    else
                    {
                        $error = $this->getErrorMsg($result->Status);
                        $history->data = $error;
                        $this->modifyOrder($order_id, $this->payment_params->invalid_status, $history, false);
                        $this->app->enqueueMessage($error, 'error');
                        $this->app->redirect($cancelUrl);
                    }
                }
                elseif($status == 'NOK')
                {
                    $this->modifyOrder($order_id, $this->payment_params->invalid_status, $history, false);
                    $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_PAYMENT_CANCELED'), 'error');
                    $this->app->redirect($cancelUrl);
                }
                else
                {
                    $this->modifyOrder($order_id, $this->payment_params->invalid_status, $history, false);
                    $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_FAILED_PAYMENT'), 'error');
                    $this->app->redirect($cancelUrl);
                }
            }
            else
            {
                $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_FAILED_PAYMENT'), 'error');
                $this->app->redirect($cancelUrl);
            }

            return false;
        }
        catch(SoapFault|Exception $e)
        {
            $this->app->enqueueMessage(JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_FAILED_PAYMENT'), 'error');
            $this->app->redirect(HIKASHOP_LIVE.'index.php?option=com_hikashop&ctrl=order&task=cancel_order&order_id=');
            return false;
        }
    }

    function writeToLog($data = null) {
        if(!empty($data)) {
            hikashop_writeToLog($data, $this->name);
            if(is_array($data) || is_object($data))
                $data = str_replace(array("\r","\n","\r\n"),"\r\n",print_r($data, true))."\r\n\r\n";
            $this->cachedDebug .= $data;
        }
        return $this->cachedDebug;
    }

    public function getPaymentDefaultValues(&$element) {
        $element->payment_name = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_NAME');
        $element->payment_description = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_DESCRIPTION');
        $element->payment_images = 'ZarinPal';
        $element->payment_params->merchantId = '';
        $element->payment_params->type = 'normal';
        $element->payment_params->test = 0;
        $element->payment_params->invalid_status = 'cancelled';
        $element->payment_params->verified_status = 'confirmed';
        $element->payment_params->debug = 0;
    }

    protected function getErrorMsg($errorCode)
    {
        $resultCodeTable = array();
        $resultCodeTable['-1']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_1');
        $resultCodeTable['-2']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_2');
        $resultCodeTable['-3']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_3');
        $resultCodeTable['-4']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_4');
        $resultCodeTable['-11']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_11');
        $resultCodeTable['-12']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_12');
        $resultCodeTable['-21']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_21');
        $resultCodeTable['-22']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_22');
        $resultCodeTable['-33']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_33');
        $resultCodeTable['-34']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_34');
        $resultCodeTable['-40']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_40');
        $resultCodeTable['-41']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_41');
        $resultCodeTable['-42']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_42');
        $resultCodeTable['-54']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_54');
        $resultCodeTable['100']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_100');
        $resultCodeTable['101']  = JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_101');

        return $resultCodeTable[$errorCode] ?? JText::_('PLG_HIKASHOPPAYMENT_TGLZARINPAL_MSG_UNKNOWN_ERROR') . $errorCode;
    }
}
